{#-
  This file was automatically generated - do not edit
-#}
{% if "material/tags" in config.plugins and tags %}
  {% include "partials/tags.html" %}
{% endif %}
{% include "partials/actions.html" %}
{% if "\x3ch1" not in page.content %}
  {% if page.meta and page.meta.anta_title %}
    <h1>{{ page.meta.anta_title | d(config.site_name, true)}}</h1>
  {% else %}
    <h1>{{ page.title | d(config.site_name, true)}}</h1>
  {% endif %}
{% endif %}
{{ page.content }}
{% include "partials/source-file.html" %}
{% include "partials/feedback.html" %}
{% include "partials/comments.html" %}
