{{ log.debug("Rendering examples section") }}

{% import "language.html" as lang with context %}

<details class="example" open>
  <summary>{{ "Examples" | convert_markdown(heading_level +1, html_id, strip_paragraph=True) }}</summary>
  {% for section_type, sub_section in section.value %}
    {% if section_type.value == "text" %}
        {{ sub_section|convert_markdown(heading_level, html_id) }}
    {% elif section_type.value == "examples" %}
        {{ sub_section|highlight(language="pycon", linenums=False) }}
    {% endif %}
  {% endfor %}
</details>
