# Copyright (c) 2024-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Integration tests for ANTA.

In particular this module test the examples/*.py scripts to make sure they are still working.
"""
